# -*- encoding: utf-8 -*-
from marshmallow_enum import EnumField

from hobbit_core.flask_hobbit.schemas import ModelSchema

from .exts import db
from . import BaseTest
from .models import User, RoleEnum


class TestSchema(BaseTest):

    def test_model_schema(self, client):

        class UserSchema(ModelSchema):
            role = EnumField(RoleEnum)

            class Meta:
                model = User

        assert UserSchema.Meta.dateformat == '%Y-%m-%d %H:%M:%S'

        user = User(username='name', email='admin@test', role=RoleEnum.admin)
        db.session.add(user)
        db.session.commit()

        data = UserSchema().dump(user).data
        assert data['role'] == {'key': 1, 'label': 'admin', 'value': '管理员'}

        class UserSchema(ModelSchema):
            role = EnumField(RoleEnum)

            class Meta:
                model = User
                verbose = False

        data = UserSchema().dump(user).data
        assert data['role'] == {'key': 1, 'value': '管理员'}

        payload = {'username': 'name', 'email': 'admin@test'}
        for role in (RoleEnum.admin.name,  RoleEnum.admin.value[0],
                     RoleEnum.admin.value[1]):
            payload['role'] = role
            assert UserSchema().load(payload).data == {
                'role': RoleEnum.admin, 'email': 'admin@test',
                'username': 'name',
            }



from flask import Blueprint, jsonify
from marshmallow import fields
from marshmallow.utils import missing
from webargs.flaskparser import use_kwargs as base_use_kwargs

from hobbit_core.flask_hobbit.utils import use_kwargs

from .schemas import UserSchema

bp = Blueprint('test', __name__)


def wrapper_kwargs(kwargs):
    res = {}
    for k, v in kwargs.items():
        if v is missing:
            res[k] = 'missing'
            continue
        res[k] = v
    return res


@bp.route('/use_kwargs_with_partial/', methods=['POST'])
@use_kwargs(UserSchema(partial=True, exclude=['role']))
def use_kwargs_with_partial(**kwargs):
    return jsonify(wrapper_kwargs(kwargs))


@bp.route('/use_kwargs_without_partial/', methods=['POST'])
@use_kwargs(UserSchema(exclude=['role']))
def use_kwargs_without_partial(**kwargs):
    return jsonify(wrapper_kwargs(kwargs))


@bp.route('/use_kwargs_dictargmap_partial/', methods=['POST'])
@use_kwargs({
    'username': fields.Str(missing=None),
    'password': fields.Str(allow_none=True),
}, schema_kwargs={'partial': True})
def use_kwargs_dictargmap_partial(**kwargs):
    print(kwargs)
    return jsonify(wrapper_kwargs(kwargs))


@bp.route('/base_use_kwargs_dictargmap_partial/', methods=['POST'])
@base_use_kwargs({
    'username': fields.Str(missing=None),
    'password': fields.Str(allow_none=True),
})
def base_use_kwargs_dictargmap_partial(**kwargs):
    return jsonify(wrapper_kwargs(kwargs))



# -*- encoding: utf-8 -*-
import enum
import six

from sqlalchemy import Integer, Column, ForeignKey, func, DateTime


class SurrogatePK(object):
    """A mixin that add ``id``、``created_at`` and ``updated_at`` columns
    to any declarative-mapped class.

    **id**: A surrogate integer 'primary key' column.

    **created_at**: Auto save ``datetime.now()`` when row created.

    **updated_at**: Auto save ``datetime.now()`` when row updated.
    """

    __table_args__ = {'extend_existing': True}

    id = Column(Integer, primary_key=True)
    created_at = Column(
        DateTime, nullable=False, server_default=func.now())
    updated_at = Column(
        DateTime, nullable=False, server_default=func.now(),
        onupdate=func.now())

    def __repr__(self):
        """You can set label property.

        Returns:
            str: ``<{classname}({pk}:{label!r})>``
        """
        return '<{classname}({pk}:{label!r})>'.format(
            classname=type(self).__name__, pk=self.id,
            label=getattr(self, 'label', ''))


def reference_col(tablename, nullable=False, pk_name='id', **kwargs):
    """Column that adds primary key foreign key reference.

    Args:
        tablename (str): Model.__table_name__.
        nullable (bool): Default is False.
        pk_name (str): Primary column's name.

    Others:

    See ``sqlalchemy.Column``

    Examples::

        from sqlalchemy.orm import relationship

        role_id = reference_col('role')
        role = relationship('Role', backref='users', cascade='all, delete')
    """

    return Column(
        ForeignKey('{0}.{1}'.format(tablename, pk_name)),
        nullable=nullable, **kwargs)


class EnumExt(enum.Enum):
    """ Extension for serialize/deserialize sqlalchemy enum field.

    Be sure ``type(key)`` is ``int`` and ``type(value)`` is ``str``
    (``label = (key, value)``).

    Examples::

        class TaskState(EnumExt):
            # label = (key, value)
            CREATED = (0, '新建')
            PENDING = (1, '等待')
            STARTING = (2, '开始')
            RUNNING = (3, '运行中')
            FINISHED = (4, '已完成')
            FAILED = (5, '失败')
    """

    @classmethod
    def strict_dump(cls, label, verbose=False):
        """Get key or value by label.

        Examples::

            TaskState.strict_dump('CREATED')  # 0
            TaskState.strict_dump('CREATED', verbose=True)  # '新建'

        Returns:
            int|str: Key or value, If label not exist, raise ``KeyError``.
        """

        return cls[label].value[1 if verbose else 0]

    @classmethod
    def dump(cls, label, verbose=False):
        """Dump one label to option.

        Examples::

            TaskState.dump('CREATED')  # {'key': 0, 'value': '新建'}

        Returns:

            dict: Dict of label's key and value. If label not exist,
            raise ``KeyError``.
        """

        ret = {'key': cls[label].value[0], 'value': cls[label].value[1]}
        if verbose:
            ret.update({'label': label})
        return ret

    @classmethod
    def load(cls, val):
        """Get label by key or value. Return val when val is label.

        Examples::

            TaskState.load('FINISHED')  # 'FINISHED'
            TaskState.load(4)  # 'FINISHED'
            TaskState.load('新建')  # 'CREATED'

        Returns:
            str|None: Label.
        """

        if val in cls.__members__:
            return val

        pos = 1 if isinstance(val, six.string_types) else 0
        for elem in cls:
            if elem.value[pos] == val:
                return elem.name

    @classmethod
    def to_opts(cls, verbose=False):
        """Enum to options.

        Examples::

            opts = TaskState.to_opts(verbose=True)
            print(opts)

            [{'key': 0, 'label': 'CREATED', 'value': u'新建'}, ...]

        Returns:
            list: List of dict which key is `key`, `value`, label.
        """

        opts = []
        for elem in cls:
            opt = {'key': elem.value[0], 'value': elem.value[1]}
            if verbose:
                opt.update({'label': elem.name})
            opts.append(opt)
        return opts



# -*- encoding: utf-8 -*-
import pytest

from hobbit_core.flask_hobbit import db

from . import BaseTest


class TestEnumExt(BaseTest):

    @pytest.fixture
    def TaskState(self):
        class _TaskState(db.EnumExt):
            CREATED = (0, u'新建')
            FINISHED = (1, u'已完成')
        return _TaskState

    def test_strict_dump(self, TaskState):
        assert 0 == TaskState.strict_dump('CREATED')
        assert u'新建' == TaskState.strict_dump('CREATED', True)

    def test_dump(self, TaskState):
        assert {'key': 0, 'value': u'新建'} == TaskState.dump('CREATED')

    def test_load(self, TaskState):
        assert 'FINISHED' == TaskState.load('FINISHED')
        assert 'FINISHED' == TaskState.load(1)
        assert 'CREATED' == TaskState.load(u'新建')
        assert TaskState.load(100) is None

    def test_to_opts(self, TaskState):
        opts = TaskState.to_opts()
        assert opts == [
            {'key': 0, 'value': u'新建'},
            {'key': 1, 'value': u'已完成'},
        ]
        opts = TaskState.to_opts(verbose=True)
        assert opts == [
            {'key': 0, 'label': 'CREATED', 'value': u'新建'},
            {'key': 1, 'label': 'FINISHED', 'value': u'已完成'},
        ]



# -*- encoding: utf-8 -*-
from hobbit_core.flask_hobbit.db import Column, SurrogatePK
from hobbit_core.flask_hobbit.db import EnumExt

from .exts import db


class RoleEnum(EnumExt):
    admin = (1, '管理员')
    normal = (2, '普通用户')


class User(SurrogatePK, db.Model):
    username = Column(db.String(50), nullable=True, unique=True)
    email = Column(db.String(50), nullable=True, unique=True)
    password = Column(db.String(255), nullable=False, server_default='')
    role = Column(db.Enum(RoleEnum), doc='角色')
